@@ -1,6 +1,6 @@
 /* block.h
  *
- * Copyright 2002 Vesa Halttunen
+ * Copyright 2002-2003 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -28,38 +28,50 @@ struct block {
   /* Name */
   char *name;
   /* Number of tracks */
-  unsigned short tracks;
+  unsigned int tracks;
   /* Number of lines */
-  unsigned short length;
+  unsigned int length;
   /* Notation data */
   unsigned char *notes;
-  /* Number of effect pages */
-  unsigned short effectpages;
-  /* Effect block array */
-  unsigned char *effects;
+  /* Number of command pages */
+  unsigned int commandpages;
+  /* Command block array */
+  unsigned char *commands;
 };
 
+/* Allocates a block */
 struct block *block_alloc(int, int, int);
 void block_free(struct block *);
-void block_set_note(struct block *, unsigned short, unsigned short,
+/* Sets a note in a block */
+void block_set_note(struct block *, unsigned int, unsigned int,
 		   unsigned char, unsigned char, unsigned char);
-void block_set_instrument(struct block *, unsigned short, unsigned short,
+/* Sets an instrument in a block */
+void block_set_instrument(struct block *, unsigned int, unsigned int,
 			  unsigned char);
-void block_set_effect(struct block *, unsigned short, unsigned short,
-		      unsigned short, unsigned char, unsigned char);
-void block_set_effect_full(struct block *, unsigned short,
-			   unsigned short, unsigned short,
-			   unsigned char, unsigned char);
-void block_set_tracks(struct block *, unsigned short);
-void block_set_length(struct block *, unsigned short);
-void block_set_effectpages(struct block *, unsigned short);
+/* Sets a part of a command in a block */
+void block_set_command(struct block *, unsigned int, unsigned int,
+		       unsigned int, unsigned char, unsigned char);
+/* Sets a command in a block */
+void block_set_command_full(struct block *, unsigned int,
+			    unsigned int, unsigned int,
+			    unsigned char, unsigned char);
+/* Sets the number of tracks in a block */
+void block_set_tracks(struct block *, unsigned int);
+/* Sets the length of a block */
+void block_set_length(struct block *, unsigned int);
+/* Sets the number of command pages in a block */
+void block_set_commandpages(struct block *, unsigned int);
 /* Copies a part of a block to a new block */
 struct block *block_copy(struct block *, int, int, int, int);
 /* Pastes a block to another block in the given position */
 void block_paste(struct block *, struct block *, int, int);
 /* Clears a part of a block */
 void block_clear(struct block *, int, int, int, int);
+/* Parses a block element in an XML file */
 struct block *block_parse(xmlDocPtr, xmlNsPtr, xmlNodePtr);
-void block_save(struct block *, xmlNodePtr);
+/* Saves a block to an XML document */
+void block_save(struct block *, int, xmlNodePtr);
+/* Transposes a block or a part of it */
+void block_transpose(struct block *, int, int, int, int, int, int);
 
 #endif
