@@ -1,6 +1,6 @@
 /* editor.h
  *
- * Copyright 2002 Vesa Halttunen
+ * Copyright 2002-2003 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -28,52 +28,47 @@
 #include <pthread.h>
 #include "song.h"
 #include "block.h"
+#include "gui.h"
+#include "player.h"
 
-#define MODE_IDLE 0
-#define MODE_PLAY_SONG 1
-#define MODE_PLAY_BLOCK 2
 #define DEFAULT_TEMPO 130
-#define TRACK_STATUS_MIDI_CHANNEL 0
-#define TRACK_STATUS_NOTE 1
-#define TRACK_STATUS_VOLUME 2
-#define TRACK_STATUS_HOLD 3
-
-/* 128 MIDI controllers plus aftertouch, channel pressure and pitch wheel */
-#define VALUES (128+3)
-#define VALUES_AFTERTOUCH 128
-#define VALUES_CHANNEL_PRESSURE 129
-#define VALUES_PITCH_WHEEL 130
 
 struct editor {
-  /* MIDI device stream pointer */
-  FILE *midi;
   /* The song currently being edited */
   struct song *song;
   /* The full file name of the song currently being edited */
   char *filename;
   /* Current location in song */
-  unsigned short section, playseq, playseqpos, block;
+  unsigned int section, playseq, position, block, line, time, cmdpage;
   /* Editor modes */
-  unsigned char octave, instrument, edit, chord, space, mode;
-  /* Status of tracks; notes playing */
-  char *trackstatus;
-  /* MIDI controller values; one for each controller on each channel */
-  unsigned char midicontrollervalues[16*VALUES];
-  /* For measuring how long the song has been playing */
-  struct timeval playingstarted, playedsofar;
-  /* The initialization status of the player thread */
-  int threadstatus;
-  /* Player thread pointer */
-  pthread_t thread;
+  unsigned int octave, instrument, edit, chord, space;
   /* Buffers for copy & paste */
   struct block *copyblock, *copytrack, *copyarea;
+  /* GUI for this editor */
+  struct gui *gui;
+  /* Player for this editor */
+  struct player *player;
 };
 
-extern struct editor editor;
-
-/* Initializes the editor and either opens an existing song or creates new */
-void editor_open(char *);
+/* Initializes the editor */
+struct editor *editor_open(char *);
+/* Opens an existing song or creates new */
+void editor_song_open(struct editor *, char *);
 /* Deinitializes the editor */
-void editor_close();
+void editor_close(struct editor *);
+
+/* Refreshes playseq from section and block from position */
+void editor_refresh_playseq_and_block(struct editor *);
+
+/* Sets the position of the editor: may only be called from the player */
+void editor_set_values(struct editor *, int, int, int, int, int, int, int);
+
+/* Sets the position of the editor: may only be called from the GUI */
+void editor_set_section_immediate(struct editor *, int);
+void editor_set_playseq_immediate(struct editor *, int);
+void editor_set_position_immediate(struct editor *, int);
+void editor_set_block_immediate(struct editor *, int);
+void editor_set_line_immediate(struct editor *, int);
+void editor_set_commandpage_immediate(struct editor *, int);
 
 #endif
