@@ -1,6 +1,6 @@
 /* instrument.c
  *
- * Copyright 2002 Vesa Halttunen
+ * Copyright 2002-2003 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -48,10 +48,11 @@ struct instrument *instrument_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
   
   if((!xmlStrcmp(cur->name, "instrument")) && (cur->ns==ns)) {
     char *prop;
+    xmlNodePtr temp=cur->xmlChildrenNode;
 
     /* Allocate instrument */
     instrument=(struct instrument *)calloc(1, sizeof(struct instrument));
-    instrument->name=xmlGetProp(cur, "name");
+    instrument->name=strdup(temp->content);
     prop=xmlGetProp(cur, "midipreset");
     if(prop!=NULL)
       instrument->midipreset=atoi(prop);
@@ -74,13 +75,15 @@ struct instrument *instrument_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur)
 }
 
 /* Saves an instrument to an XML document */
-void instrument_save(struct instrument *instrument, xmlNodePtr parent) {
+void instrument_save(struct instrument *instrument, int number,
+		     xmlNodePtr parent) {
   char c[10];
 
   xmlNodePtr node, lb;
 
-  node=xmlNewChild(parent, NULL, "instrument", NULL);
-  xmlSetProp(node, "name", instrument->name);
+  node=xmlNewChild(parent, NULL, "instrument", instrument->name);
+  snprintf(c, 10, "%d", number);
+  xmlSetProp(node, "number", c);
   snprintf(c, 10, "%d", instrument->midipreset);
   xmlSetProp(node, "midipreset", c);
   snprintf(c, 10, "%d", instrument->midichannel);
