@@ -1,6 +1,6 @@
 /* main.c
  *
- * Copyright 2002 Vesa Halttunen
+ * Copyright 2002-2003 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -23,38 +23,51 @@
 #  include <config.h>
 #endif
 
-#include <gnome.h>
-
+#include <libgnome/libgnome.h>
+#include <glade/glade.h>
 #include <glib.h>
-#include "editor.h"
-#include "callbacks.h"
+#include <unistd.h>
 
-extern GtkWidget *gui_window_main;
+#include "editor.h"
+#include "gui.h"
+
+extern gboolean gui_quit;
 
 int main(int argc, char *argv[]) {
-#ifdef G_THREADS_IMPL_POSIX
+  struct editor *editor=NULL;
 
-  if(!g_thread_supported())
+  if(!g_thread_supported()) {
     g_thread_init(NULL);
+    gdk_threads_init();
+  } else {
+    fprintf(stderr, "No threads implementation. Tutka requires threads.\n");
+    return -1;
+  }
+
   gnome_init("Tutka", VERSION, argc, argv);
+  glade_gnome_init();
 
-  if(argc>1)
-    editor_open(argv[1]);
-  else
-    editor_open(NULL);
+  editor=editor_open(PACKAGE_DATA_DIR "/glade/tutka.glade");
 
-  gui_open();
+  if(editor!=NULL) {
+    /* Open a song in the editor */
+    if(argc>1)
+      editor_song_open(editor, argv[1]);
+    else
+      editor_song_open(editor, NULL);
+    
+    /* Main loop */
+    gdk_threads_enter();
+    while(!gui_quit) {
+      while(gtk_events_pending())
+	gtk_main_iteration_do(FALSE);
+      gui_refresh(editor->gui);
+      usleep(40);
+    }
+    gdk_threads_leave();
 
-  gdk_threads_enter();
-  gtk_main();
-  gdk_threads_leave();
-
-  editor_close();
+    editor_close(editor);
+  }
 
   return 0;
-#else
-  fprintf(stderr, "No POSIX threads implementation. Tutka needs to be compiled against one.\n");
-
-  return -1;
-#endif
 }
