@@ -1,6 +1,6 @@
 /* song.c
  *
- * Copyright 2002 Vesa Halttunen
+ * Copyright 2002-2003 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -23,12 +23,6 @@
 #include <string.h>
 #include <libxml/tree.h>
 #include "song.h"
-#include "editor.h"
-#include "player.h"
-
-char *xmlencode(unsigned char *, int);
-
-extern struct editor editor;
 
 /* Allocates a new song structure and initializes it to default values */
 struct song *song_alloc(void) {
@@ -40,7 +34,7 @@ struct song *song_alloc(void) {
   song->tempo=130;
   song->ticksperline=6;
   song->numsections=1;
-  song->sections=(unsigned short *)calloc(1, sizeof(unsigned short));
+  song->sections=(unsigned int *)calloc(1, sizeof(unsigned int));
   song->numplayseqs=1;
   song->playseqs=(struct playseq **)calloc(1, sizeof(struct playseq *));
   song->playseqs[0]=playseq_alloc();
@@ -51,7 +45,7 @@ struct song *song_alloc(void) {
   song->instruments=(struct instrument **)calloc(1, sizeof(struct instrument *));
   song->instruments[0]=instrument_alloc();
   song->maxtracks=4;
-  song->trackvolumes=(char *)calloc(4, sizeof(char));
+  song->trackvolumes=(unsigned int *)calloc(4, sizeof(unsigned int));
   song->mastervolume=127;
   for(i=0; i<song->maxtracks; i++)
     song->trackvolumes[i]=127;
@@ -61,52 +55,60 @@ struct song *song_alloc(void) {
 
 /* Frees a song structure and its contents */
 void song_free(struct song *song) {
-  if(song) {
-    if(song->name)
-      free(song->name);
-    
-    if(song->numsections>0)
-      free(song->sections);
-    
-    if(song->numplayseqs>0) {
-      int i;
-      
-      for(i=0; i<song->numplayseqs; i++)
-	playseq_free(song->playseqs[i]);
-      
-      free(song->playseqs);
-    }
-    
-    if(song->numblocks>0) {
-      int i;
-      
-      for(i=0; i<song->numblocks; i++)
-	block_free(song->blocks[i]);
-      
-      free(song->blocks);
-    }
-    
-    if(song->numinstruments>0) {
-      int i;
-      
-      for(i=0; i<song->numinstruments; i++)
-	if(song->instruments[i]!=NULL)
-	  instrument_free(song->instruments[i]);
-      
-      free(song->instruments);
-    }
-    
-    if(song->maxtracks>0)
-      free(song->trackvolumes);
-    
-    free(song);
+  if(song==NULL) {
+    fprintf(stderr, "song_free() called with null song\n");
+    return;
   }
+
+  if(song->name)
+    free(song->name);
+    
+  if(song->numsections>0)
+    free(song->sections);
+    
+  if(song->numplayseqs>0) {
+    int i;
+      
+    for(i=0; i<song->numplayseqs; i++)
+      playseq_free(song->playseqs[i]);
+      
+    free(song->playseqs);
+  }
+    
+  if(song->numblocks>0) {
+    int i;
+      
+    for(i=0; i<song->numblocks; i++)
+      block_free(song->blocks[i]);
+      
+    free(song->blocks);
+  }
+    
+  if(song->numinstruments>0) {
+    int i;
+      
+    for(i=0; i<song->numinstruments; i++)
+      if(song->instruments[i]!=NULL)
+	instrument_free(song->instruments[i]);
+      
+    free(song->instruments);
+  }
+    
+  if(song->maxtracks>0)
+    free(song->trackvolumes);
+    
+  free(song);
 }
 
 /* Inserts a new block in the block array in the given position */
-void song_insert_block(struct song *song, int pos) {
+void song_insert_block(struct song *song, int pos, int current) {
   int i;
 
+  if(song==NULL) {
+    fprintf(stderr, "song_insert_block() called with null song\n");
+    return;
+  }
+
   /* Check block existence */
   if(pos<0)
     pos=0;
@@ -122,9 +124,9 @@ void song_insert_block(struct song *song, int pos) {
     song->blocks[i]=song->blocks[i-1];
 
   /* Insert a new block similar to the current block */
-  song->blocks[pos]=block_alloc(song->blocks[editor.block]->tracks,
-				song->blocks[editor.block]->length,
-				song->blocks[editor.block]->effectpages);
+  song->blocks[pos]=block_alloc(song->blocks[current]->tracks,
+				song->blocks[current]->length,
+				song->blocks[current]->commandpages);
   
   /* Update playing sequences */
   for(i=0; i<song->numplayseqs; i++) {
@@ -140,6 +142,11 @@ void song_insert_block(struct song *song, int pos) {
 void song_delete_block(struct song *song, int pos) {
   int i;
 
+  if(song==NULL) {
+    fprintf(stderr, "song_delete_block() called with null song\n");
+    return;
+  }
+  
   /* Check block existence */
   if(pos<0)
     pos=0;
@@ -175,6 +182,11 @@ void song_delete_block(struct song *song, int pos) {
 void song_insert_playseq(struct song *song, int pos) {
   int i;
 
+  if(song==NULL) {
+    fprintf(stderr, "song_insert_playseq() called with null song\n");
+    return;
+  }
+
   /* Check playseq existence */
   if(pos<0)
     pos=0;
@@ -203,6 +215,11 @@ void song_insert_playseq(struct song *song, int pos) {
 void song_delete_playseq(struct song *song, int pos) {
   int i;
   
+  if(song==NULL) {
+    fprintf(stderr, "song_delete_playseq() called with null song\n");
+    return;
+  }
+
   /* Check block existence */
   if(pos<0)
     pos=0;
@@ -227,8 +244,6 @@ void song_delete_playseq(struct song *song, int pos) {
       if(song->sections[i]>=pos && song->sections[i]>0)
 	  song->sections[i]--;
     }
-
-    player_refresh_playseq_and_block();
   }
 }
 
@@ -236,25 +251,44 @@ void song_delete_playseq(struct song *song, int pos) {
 /* Inserts a new section in the section array in the given position */
 void song_insert_section(struct song *song, int pos) {
   int i;
+  unsigned int old;
 
-  /* Check block existence */
+  if(song==NULL) {
+    fprintf(stderr, "song_insert_section() called with null song\n");
+    return;
+  }
+
+  /* Check that the value is possible */
   if(pos<0)
     pos=0;
   else if(pos>song->numsections)
     pos=song->numsections;
 
+  /* Which playing sequence number to insert */
+  if(pos<song->numsections)
+    old=song->sections[pos];
+  else
+    old=song->sections[song->numsections-1];
+
   song->numsections++;
   /* Reallocate the sections array */
-  song->sections=(unsigned short *)realloc(song->sections, song->numsections*sizeof(unsigned short));
+  song->sections=(unsigned int *)realloc(song->sections, song->numsections*sizeof(unsigned int));
 
   /* Move rest of the sections onwards */
   for(i=song->numsections-1; i>pos; i--)
     song->sections[i]=song->sections[i-1];
+
+  song->sections[pos]=old;
 }
 
 /* Deletes a section from the given position of the section array */
 void song_delete_section(struct song *song, int pos) {
   int i;
+
+  if(song==NULL) {
+    fprintf(stderr, "song_delete_section() called with null song\n");
+    return;
+  }
   
   /* Check block existence */
   if(pos<0)
@@ -271,9 +305,7 @@ void song_delete_section(struct song *song, int pos) {
       song->sections[i]=song->sections[i+1];
     
     /* Reallocate the sections array */
-    song->sections=(unsigned short *)realloc(song->sections, song->numsections*sizeof(unsigned short));
-
-    player_refresh_playseq_and_block();
+    song->sections=(unsigned int *)realloc(song->sections, song->numsections*sizeof(unsigned int));
   }
 }
 
@@ -281,6 +313,11 @@ void song_delete_section(struct song *song, int pos) {
 void song_insert_sysex(struct song *song, int pos) {
   int i;
 
+  if(song==NULL) {
+    fprintf(stderr, "song_insert_sysex() called with null song\n");
+    return;
+  }
+
   /* Check sysex existence */
   if(pos<0)
     pos=0;
@@ -303,6 +340,11 @@ void song_insert_sysex(struct song *song, int pos) {
 void song_delete_sysex(struct song *song, int pos) {
   int i;
 
+  if(song==NULL) {
+    fprintf(stderr, "song_delete_sysex() called with null song\n");
+    return;
+  }
+
   /* Check sysex existence */
   if(pos<0)
     pos=0;
@@ -330,23 +372,45 @@ void song_delete_sysex(struct song *song, int pos) {
 
 /* Sets a section in the given position to point somewhere */
 void song_set_section(struct song *song, int pos, int playseq) {
+  if(song==NULL) {
+    fprintf(stderr, "song_set_section() called with null song\n");
+    return;
+  }
+
   if(playseq>=0 && playseq<song->numplayseqs)
     song->sections[pos]=playseq;
 }
 
 /* Sets the number of ticks per line for a song */
-void song_set_tpl(struct song *song, unsigned char ticksperline) {
+void song_set_tpl(struct song *song, unsigned int ticksperline) {
+  if(song==NULL) {
+    fprintf(stderr, "song_set_tpl() called with null song\n");
+    return;
+  }
+
   song->ticksperline=ticksperline;
 }
 
 /* Sets the tempo of a song */
-void song_set_tempo(struct song *song, unsigned char tempo) {
+void song_set_tempo(struct song *song, unsigned int tempo) {
+  if(song==NULL) {
+    fprintf(stderr, "song_set_tempo() called with null song\n");
+    return;
+  }
+
   song->tempo=tempo;
 }
 
 /* If the maximum number of tracks has changed recreate the track volumes */
 int song_check_maxtracks(struct song *song) {
-  int i, oldmax=song->maxtracks, max=0;
+  int i, oldmax, max=0;
+
+  if(song==NULL) {
+    fprintf(stderr, "song_check_maxtracks() called with null song\n");
+    return 0;
+  }
+
+  oldmax=song->maxtracks;
 
   /* Check the maximum number of tracks; */
   for(i=0; i<song->numblocks; i++)
@@ -356,24 +420,13 @@ int song_check_maxtracks(struct song *song) {
   /* If the maximum number of tracks as changed... */
   if(oldmax!=max) {
     /* Reallocate volume array */
-    song->trackvolumes=(unsigned char *)realloc(song->trackvolumes,
-						max*sizeof(unsigned char));
+    song->trackvolumes=(unsigned int *)realloc(song->trackvolumes,
+					       max*sizeof(unsigned int));
 
     /* Set new tracks to volume 127 */
     for(i=song->maxtracks; i<max; i++)
       song->trackvolumes[i]=127;
 
-    /* Reallocate track status array */
-    editor.trackstatus=(char *)realloc(editor.trackstatus, 4*max*sizeof(char));
-
-    /* Set new tracks to -1 */
-    for(i=song->maxtracks; i<max; i++) {
-      editor.trackstatus[i*4]=-1;
-      editor.trackstatus[i*4+1]=-1;
-      editor.trackstatus[i*4+2]=-1;
-      editor.trackstatus[i*4+3]=-1;
-    }
-
     song->maxtracks=max;
     return 1;
   } else
@@ -382,6 +435,11 @@ int song_check_maxtracks(struct song *song) {
 
 /* Make sure the instrument exists; add instruments if necessary */
 void song_check_instrument(struct song *song, int instrument) {
+  if(song==NULL) {
+    fprintf(stderr, "song_check_instrument() called with null song\n");
+    return;
+  }
+
   if(instrument>=song->numinstruments || song->instruments[instrument]==NULL) {
     if(instrument>=song->numinstruments) {
       int i;
@@ -400,46 +458,46 @@ void song_check_instrument(struct song *song, int instrument) {
 struct song *song_convert_MMD2(struct MMD2 *mmd) {
   struct song *song;
   int i, j, k, l;
-  unsigned char effectconversion[256];
+  unsigned char commandconversion[256];
 
   if(mmd==NULL)
     return NULL;
 
-  /* Create a table for converting MMD effects; values need special handling */
+  /* Create a table for converting MMD commands; values need special handling */
   for(i=0; i<256; i++)
-    effectconversion[i]=i;
-  effectconversion[0]=EFFECT_MIDI_COMMAND_VALUE;
-  effectconversion[1]=EFFECT_PITCH_WHEEL;
-  effectconversion[2]=EFFECT_PITCH_WHEEL;
-  effectconversion[3]=EFFECT_PITCH_WHEEL;
-  effectconversion[4]=EFFECT_NOT_DEFINED;
-  effectconversion[5]=EFFECT_NOT_DEFINED;
-  effectconversion[6]=EFFECT_NOT_DEFINED;
-  effectconversion[7]=EFFECT_NOT_DEFINED;
-  effectconversion[8]=EFFECT_NOT_DEFINED;
-  effectconversion[9]=EFFECT_TPL;
-  effectconversion[10]=EFFECT_VELOCITY;
-  effectconversion[11]=EFFECT_PLAYSEQ_POSITION;
-  effectconversion[12]=EFFECT_VELOCITY;
-  effectconversion[13]=EFFECT_CHANNEL_PRESSURE;
-  effectconversion[14]=EFFECT_NOT_DEFINED;
-  effectconversion[15]=EFFECT_TEMPO;
-  effectconversion[16]=EFFECT_SEND_SYSEX;
-  effectconversion[17]=EFFECT_NOT_DEFINED;
-  effectconversion[18]=EFFECT_NOT_DEFINED;
-  effectconversion[19]=EFFECT_NOT_DEFINED;
-  effectconversion[20]=EFFECT_NOT_DEFINED;
-  effectconversion[21]=EFFECT_NOT_DEFINED;
-  effectconversion[22]=EFFECT_NOT_DEFINED;
-  effectconversion[23]=EFFECT_NOT_DEFINED;
-  effectconversion[24]=EFFECT_NOT_DEFINED;
-  effectconversion[25]=EFFECT_NOT_DEFINED;
-  effectconversion[26]=EFFECT_NOT_DEFINED;
-  effectconversion[27]=EFFECT_NOT_DEFINED;
-  effectconversion[28]=EFFECT_NOT_DEFINED;
-  effectconversion[29]=EFFECT_NOT_DEFINED;
-  effectconversion[30]=EFFECT_NOT_DEFINED;
-  effectconversion[31]=EFFECT_DELAY;
+    commandconversion[i]=i;
+  commandconversion[0]=COMMAND_MIDI_COMMAND_VALUE;
+  commandconversion[1]=COMMAND_PITCH_WHEEL;
+  commandconversion[2]=COMMAND_PITCH_WHEEL;
+  commandconversion[3]=COMMAND_PITCH_WHEEL;
+  commandconversion[4]=COMMAND_NOT_DEFINED;
+  commandconversion[5]=COMMAND_NOT_DEFINED;
+  commandconversion[6]=COMMAND_NOT_DEFINED;
+  commandconversion[7]=COMMAND_NOT_DEFINED;
+  commandconversion[8]=COMMAND_NOT_DEFINED;
+  commandconversion[9]=COMMAND_TPL;
+  commandconversion[10]=COMMAND_VELOCITY;
+  commandconversion[11]=COMMAND_PLAYSEQ_POSITION;
+  commandconversion[12]=COMMAND_VELOCITY;
+  commandconversion[13]=COMMAND_CHANNEL_PRESSURE;
+  commandconversion[14]=COMMAND_NOT_DEFINED;
+  commandconversion[15]=COMMAND_TEMPO;
+  commandconversion[16]=COMMAND_SEND_SYSEX;
+  commandconversion[17]=COMMAND_NOT_DEFINED;
+  commandconversion[18]=COMMAND_NOT_DEFINED;
+  commandconversion[19]=COMMAND_NOT_DEFINED;
+  commandconversion[20]=COMMAND_NOT_DEFINED;
+  commandconversion[21]=COMMAND_NOT_DEFINED;
+  commandconversion[22]=COMMAND_NOT_DEFINED;
+  commandconversion[23]=COMMAND_NOT_DEFINED;
+  commandconversion[24]=COMMAND_NOT_DEFINED;
+  commandconversion[25]=COMMAND_NOT_DEFINED;
+  commandconversion[26]=COMMAND_NOT_DEFINED;
+  commandconversion[27]=COMMAND_NOT_DEFINED;
+  commandconversion[28]=COMMAND_NOT_DEFINED;
+  commandconversion[29]=COMMAND_NOT_DEFINED;
+  commandconversion[30]=COMMAND_NOT_DEFINED;
+  commandconversion[31]=COMMAND_DELAY;
 
   song=(struct song *)calloc(1, sizeof(struct song));
   if(mmd->expdata && mmd->expdata->songname)
@@ -452,8 +510,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
     song->ticksperline=mmd->song->tempo2;
     /* Sections */
     song->numsections=mmd->song->songlen;
-    song->sections=(unsigned short *)calloc(song->numsections,
-					    sizeof(unsigned short));
+    song->sections=(unsigned int *)calloc(song->numsections,
+					    sizeof(unsigned int));
     for(i=0; i<song->numsections; i++) {
       /* FIX: Where to get these? */
     }
@@ -469,8 +527,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
       if(PlaySeq->name)
 	playseq->name=(char *)strdup(PlaySeq->name);
       playseq->length=PlaySeq->length;
-      playseq->blocknumbers=(unsigned short *)calloc(playseq->length,
-						     sizeof(unsigned short));
+      playseq->blocknumbers=(unsigned int *)calloc(playseq->length,
+						     sizeof(unsigned int));
       for(j=0; j<song->playseqs[i]->length; j++)
 	playseq->blocknumbers[j]=PlaySeq->seq[j];
     }
@@ -493,21 +551,21 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 	for(k=0; k<block->tracks; k++) {
 	  block->notes[(j*block->tracks+k)*2]=notes[(j*block->tracks+k)*4];
 	  block->notes[(j*block->tracks+k)*2+1]=notes[(j*block->tracks+k)*4+1];
-	  block->effects[(j*block->tracks+k)*2]=effectconversion[notes[(j*block->tracks+k)*4+2]];
-	  block->effects[(j*block->tracks+k)*2+1]=notes[(j*block->tracks+k)*4+3];
-	  /* Effect values may need special handling */
-	  song_convert_effects(&(block->effects[(j*block->tracks+k)*2]));
+	  block->commands[(j*block->tracks+k)*2]=commandconversion[notes[(j*block->tracks+k)*4+2]];
+	  block->commands[(j*block->tracks+k)*2+1]=notes[(j*block->tracks+k)*4+3];
+	  /* Command values may need special handling */
+	  song_convert_commands(&(block->commands[(j*block->tracks+k)*2]));
 	}
 
       if(MMD1Block->info->pagetable) {
 	for(j=0; j<MMD1Block->info->pagetable->num_pages; j++) {
 	  for(k=0; k<(MMD1Block->lines+1); k++)
 	    for(l=0; l<MMD1Block->numtracks; l++) {
-	      block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]=effectconversion[MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2]];
-	      block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]=MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2+1];
+	      block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]=commandconversion[MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2]];
+	      block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2+1]=MMD1Block->info->pagetable->page[j][(k*MMD1Block->numtracks+l)*2+1];
 
-	      /* Effect values may need special handling */
-	      song_convert_effects(&(block->effects[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]));
+	      /* Command values may need special handling */
+	      song_convert_commands(&(block->commands[(j+1)*2*block->tracks*block->length+(k*block->tracks+l)*2]));
 	    }
 	}
       }
@@ -533,7 +591,8 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
 
     /* Track volumes */
     song->maxtracks=mmd->song->numtracks;
-    song->trackvolumes=(char *)calloc(song->maxtracks, sizeof(char));
+    song->trackvolumes=(unsigned int *)calloc(song->maxtracks,
+					      sizeof(unsigned int));
     for(i=0; i<song->maxtracks; i++) {
       if(mmd->song->trackvols[i]<64)
 	song->trackvolumes[i]=mmd->song->trackvols[i]*2;
@@ -575,65 +634,79 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
   return song;
 }
 
-/* Converts MMD effect values at given location */
-void song_convert_effects(unsigned char *effect) {
-  switch(effect[0]) {
-  case EFFECT_VELOCITY:
+/* Transposes a song */
+void song_transpose(struct song *song, int instruments, int halfnotes) {
+  int i;
+
+  if(song==NULL) {
+    fprintf(stderr, "song_transpose() called with null song\n");
+    return;
+  }
+
+  for(i=0; i<song->numblocks; i++)
+    block_transpose(song->blocks[i], instruments, halfnotes, 0, 0,
+		    song->blocks[i]->tracks-1, song->blocks[i]->length-1);
+}
+
+/* Converts MMD command values at given location */
+void song_convert_commands(unsigned char *command) {
+  switch(command[0]) {
+  case COMMAND_VELOCITY:
     /* Fix volume range to 0..127 */
-    if(effect[1]==64)
-      effect[1]=127;
+    if(command[1]==64)
+      command[1]=127;
     else
-      effect[1]*=2;
+      command[1]*=2;
     break;
-  case EFFECT_DELAY:
+  case COMMAND_DELAY:
     /* Command 1F in MMD is both note delay and retrigger */
-    if(effect[1]>15)
-      effect[1]=effect[1]>>4;
+    if(command[1]>15)
+      command[1]=command[1]>>4;
     else
-      effect[0]=EFFECT_RETRIGGER;
+      command[0]=COMMAND_RETRIGGER;
     break;
-  case EFFECT_TEMPO:
+  case COMMAND_TEMPO:
     /* Values >240 have special meanings in OctaMED */
-    if(effect[1]>240) {
-      switch(effect[1]) {
+    if(command[1]>240) {
+      switch(command[1]) {
       case 241:
 	/* Makes a single note play twice */
-	effect[0]=EFFECT_RETRIGGER;
-	effect[1]=3;
+	command[0]=COMMAND_RETRIGGER;
+	command[1]=3;
 	break;
       case 242:
 	/* Delays the start of a note by half a line */
-	effect[0]=EFFECT_DELAY;
-	effect[1]=3;
+	command[0]=COMMAND_DELAY;
+	command[1]=3;
 	break;
       case 243:
 	/* Works like 0FF1 except the note is played three times */
-	effect[0]=EFFECT_RETRIGGER;
-	effect[1]=2;
+	command[0]=COMMAND_RETRIGGER;
+	command[1]=2;
 	break;
       case 244:
 	/* Delays the note one-third of a line */
-	effect[0]=EFFECT_DELAY;
-	effect[1]=2;
+	command[0]=COMMAND_DELAY;
+	command[1]=2;
 	break;
       case 245:
 	/* Delays the note two thirds of a line */
-	effect[0]=EFFECT_DELAY;
-	effect[1]=4;
+	command[0]=COMMAND_DELAY;
+	command[1]=4;
 	break;
       case 254:
 	/* Stops the song playing */
-	effect[0]=EFFECT_STOP;
-	effect[1]=0;
+	command[0]=COMMAND_STOP;
+	command[1]=0;
 	break;
       case 255:
 	/* Stops the note on the current track */
-	effect[0]=EFFECT_VELOCITY;
-	effect[1]=0;
+	command[0]=COMMAND_VELOCITY;
+	command[1]=0;
 	break;
       default:
-	effect[0]=0;
-	effect[1]=0;
+	command[0]=0;
+	command[1]=0;
 	break;
       }
     }
@@ -699,108 +772,205 @@ struct song *song_parse(xmlDocPtr doc, xmlNsPtr ns, xmlNodePtr cur) {
     prop=xmlGetProp(cur, "mastervolume");
     if(prop!=NULL)
       song->mastervolume=atoi(prop);
+    prop=xmlGetProp(cur, "sendsync");
+    if(prop!=NULL)
+      song->sendsync=atoi(prop);
 
     cur=cur->xmlChildrenNode;
     while(cur!=NULL) {
       if((!xmlStrcmp(cur->name, "blocks")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
-
 	/* Parse and add all block elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
+
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
+	    int number=-1;
 	    struct block *block=block_parse(doc, ns, temp);
+
 	    if(block!=NULL) {
-	      song->numblocks++;
-	      song->blocks=realloc(song->blocks,
-				   song->numblocks*sizeof(struct block *));
-	      song->blocks[song->numblocks-1]=block;
+	      prop=xmlGetProp(temp, "number");
+	      if(prop!=NULL)
+		number=atoi(prop);
+	      
+	      if(number>=song->numblocks) {
+		int i;
+		
+		song->blocks=realloc(song->blocks,
+				     (number+1)*sizeof(struct block *));
+		for(i=song->numblocks; i<=number; i++)
+		  song->blocks[i]=NULL;
+	      }
+	      if(song->blocks[number]!=NULL)
+		block_free(song->blocks[number]);
+	      song->blocks[number]=block;
+	      song->numblocks=number+1;
 	    }
 	  }
 	  
 	  temp=temp->next;
 	}
       } else if((!xmlStrcmp(cur->name, "sections")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
-
 	/* Parse and add all section elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
+
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
-	    prop=xmlGetProp(temp, "playseq");
-	    if(prop!=NULL) {
-	      song->numsections++;
-	      song->sections=realloc(song->sections, song->numsections*sizeof(unsigned short));
-	      song->sections[song->numsections-1]=atoi(prop);
-	    }
+	    int number=-1;
+
+	    /* The section number is required */
+	    if((!xmlStrcmp(temp->name, "section")) && (temp->ns==ns)) {
+	      xmlNodePtr temp2;
+	      prop=xmlGetProp(temp, "number");
+	      if(prop!=NULL)
+		number=atoi(prop);
+
+	      if(number>=song->numsections) {
+		int i;
+		
+		song->sections=realloc(song->sections,
+				       (number+1)*sizeof(unsigned int));
+		for(i=song->numsections; i<=number; i++)
+		  song->sections[i]=0;
+	      }
+
+	      /* Get playing sequence */
+	      temp2=temp->xmlChildrenNode;
+	      if(temp2!=NULL)
+		song->sections[number]=atoi(temp2->content);
+	      song->numsections=number+1;
+	    } else
+	      fprintf(stderr, "XML error: expected section, got %s\n",
+		      temp->name);
 	  }
 	  
 	  temp=temp->next;
 	}
-      } else if((!xmlStrcmp(cur->name, "playseqs")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
+      } else if((!xmlStrcmp(cur->name, "playingsequences")) && (cur->ns==ns)) {
+	/* Parse and add all playingsequence elements */
 
-	/* Parse and add all playseq elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
+	    int number=-1;
 	    struct playseq *playseq=playseq_parse(doc, ns, temp);
+
 	    if(playseq!=NULL) {
-	      song->numplayseqs++;
-	      song->playseqs=realloc(song->playseqs, song->numplayseqs*sizeof(struct playseq *));
-	      song->playseqs[song->numplayseqs-1]=playseq;
+	      prop=xmlGetProp(temp, "number");
+	      if(prop!=NULL)
+		number=atoi(prop);
+	      
+	      if(number>=song->numplayseqs) {
+		int i;
+		
+		song->playseqs=realloc(song->playseqs,
+				       (number+1)*sizeof(struct playseq *));
+		for(i=song->numplayseqs; i<=number; i++)
+		  song->playseqs[i]=NULL;
+	      }
+	      if(song->playseqs[number]!=NULL)
+		playseq_free(song->playseqs[number]);
+	      song->playseqs[number]=playseq;
+	      song->numplayseqs=number+1;
 	    }
 	  }
 	  
 	  temp=temp->next;
 	}
       } else if((!xmlStrcmp(cur->name, "instruments")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
-
 	/* Parse and add all instrument elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
+
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
+	    int number=-1;
 	    struct instrument *instrument=instrument_parse(doc, ns, temp);
+
 	    if(instrument!=NULL) {
-	      song->numinstruments++;
-	      song->instruments=realloc(song->instruments, song->numinstruments*sizeof(struct instrument *));
-	      song->instruments[song->numinstruments-1]=instrument;
+	      prop=xmlGetProp(temp, "number");
+	      if(prop!=NULL)
+		number=atoi(prop);
+	      
+	      if(number>=song->numinstruments) {
+		int i;
+		
+		song->instruments=realloc(song->instruments,
+				       (number+1)*sizeof(struct instrument *));
+		for(i=song->numinstruments; i<=number; i++)
+		  song->instruments[i]=NULL;
+	      }
+	      if(song->instruments[number]!=NULL)
+		instrument_free(song->instruments[number]);
+	      song->instruments[number]=instrument;
+	      song->numinstruments=number+1;
 	    }
 	  }
 	  
 	  temp=temp->next;
 	}
       } else if((!xmlStrcmp(cur->name, "trackvolumes")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
-
 	/* Parse and add all track volume elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
+
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
-	    prop=xmlGetProp(temp, "volume");
-	    if(prop!=NULL) {
-	      song->maxtracks++;
-	      song->trackvolumes=realloc(song->trackvolumes,
-				     song->maxtracks*sizeof(unsigned char));
-	      song->trackvolumes[song->maxtracks-1]=atoi(prop);
-	    }
+	    int track=-1;
+
+	    /* The track number is required */
+	    if((!xmlStrcmp(temp->name, "trackvolume")) && (temp->ns==ns)) {
+	      xmlNodePtr temp2;
+	      prop=xmlGetProp(temp, "track");
+	      if(prop!=NULL)
+		track=atoi(prop);
+
+	      if(track>=song->maxtracks) {
+		int i;
+		
+		song->trackvolumes=realloc(song->trackvolumes,
+					   (track+1)*sizeof(unsigned int));
+		for(i=song->maxtracks; i<=track; i++)
+		  song->trackvolumes[i]=0;
+	      }
+
+	      /* Get the volume */
+	      temp2=temp->xmlChildrenNode;
+	      if(temp2!=NULL)
+		song->trackvolumes[track]=atoi(temp2->content);
+	      song->maxtracks=track+1;
+	    } else 
+	      fprintf(stderr, "XML error: expected section, got %s\n",
+		      temp->name);
 	  }
 	  
 	  temp=temp->next;
 	}
       } else if((!xmlStrcmp(cur->name, "sysexes")) && (cur->ns==ns)) {
-	xmlNodePtr temp;
-
 	/* Parse and add all SysEx elements */
-	temp=cur->xmlChildrenNode;
+	xmlNodePtr temp=cur->xmlChildrenNode;
+
 	while(temp!=NULL) {
 	  if(!(xmlIsBlankNode(temp))) {
+	    int number=-1;
 	    struct sysex *sysex=sysex_parse(doc, ns, temp);
+
 	    if(sysex!=NULL) {
-	      song->numsysexes++;
-	      song->sysexes=realloc(song->sysexes, song->numsysexes*sizeof(struct sysex *));
-	      song->sysexes[song->numsysexes-1]=sysex;
+	      prop=xmlGetProp(temp, "number");
+	      if(prop!=NULL)
+		number=atoi(prop);
+	      
+	      if(number>=song->numsysexes) {
+		int i;
+		
+		song->sysexes=realloc(song->sysexes,
+				       (number+1)*sizeof(struct sysex *));
+		for(i=song->numsysexes; i<=number; i++)
+		  song->sysexes[i]=NULL;
+	      }
+	      if(song->sysexes[number]!=NULL) {
+		free(song->sysexes[number]->data);
+		free(song->sysexes[number]);
+	      }
+	      song->sysexes[number]=sysex;
+	      song->numsysexes=number+1;
 	    }
 	  }
 	  
@@ -822,6 +992,11 @@ void song_save(struct song *song, char *filename) {
   int i;
   char c[10];
 
+  if(song==NULL) {
+    fprintf(stderr, "song_save() called with null song\n");
+    return;
+  }
+
   doc=xmlNewDoc("1.0");
   doc->xmlRootNode=xmlNewDocNode(doc, NULL, "song", NULL);
   xmlSetProp(doc->xmlRootNode, "name", song->name);
@@ -831,6 +1006,8 @@ void song_save(struct song *song, char *filename) {
   xmlSetProp(doc->xmlRootNode, "ticksperline", c);
   snprintf(c, 10, "%d", song->mastervolume);
   xmlSetProp(doc->xmlRootNode, "mastervolume", c);
+  snprintf(c, 10, "%d", song->sendsync);
+  xmlSetProp(doc->xmlRootNode, "sendsync", c);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
@@ -839,7 +1016,7 @@ void song_save(struct song *song, char *filename) {
   xmlAddChild(node, lb);
   /* Add all blocks */
   for(i=0; i<song->numblocks; i++)
-    block_save(song->blocks[i], node);
+    block_save(song->blocks[i], i, node);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
@@ -848,21 +1025,22 @@ void song_save(struct song *song, char *filename) {
   xmlAddChild(node, lb);
   /* Add all sections */
   for(i=0; i<song->numsections; i++) {
-    subnode=xmlNewChild(node, NULL, "section", NULL);
     snprintf(c, 10, "%d", song->sections[i]);
-    xmlSetProp(subnode, "playseq", c);
+    subnode=xmlNewChild(node, NULL, "section", c);
+    snprintf(c, 10, "%d", i);
+    xmlSetProp(subnode, "number", c);
     lb=xmlNewText("\n");
     xmlAddChild(node, lb);
   }
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
-  node=xmlNewChild(doc->xmlRootNode, NULL, "playseqs", NULL);
+  node=xmlNewChild(doc->xmlRootNode, NULL, "playingsequences", NULL);
   lb=xmlNewText("\n");
   xmlAddChild(node, lb);
   /* Add all playing sequences */
   for(i=0; i<song->numplayseqs; i++)
-    playseq_save(song->playseqs[i], node);
+    playseq_save(song->playseqs[i], i, node);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
@@ -871,7 +1049,7 @@ void song_save(struct song *song, char *filename) {
   xmlAddChild(node, lb);
   /* Add all instruments */
   for(i=0; i<song->numinstruments; i++)
-    instrument_save(song->instruments[i], node);
+    instrument_save(song->instruments[i], i, node);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
@@ -880,9 +1058,10 @@ void song_save(struct song *song, char *filename) {
   xmlAddChild(node, lb);
   /* Add all track volumes */
   for(i=0; i<song->maxtracks; i++) {
-    subnode=xmlNewChild(node, NULL, "track", NULL);
     snprintf(c, 10, "%d", song->trackvolumes[i]);
-    xmlSetProp(subnode, "volume", c);
+    subnode=xmlNewChild(node, NULL, "trackvolume", c);
+    snprintf(c, 10, "%d", i);
+    xmlSetProp(subnode, "track", c);
     lb=xmlNewText("\n");
     xmlAddChild(node, lb);
   }
@@ -894,7 +1073,7 @@ void song_save(struct song *song, char *filename) {
   xmlAddChild(node, lb);
   /* Add all SysEx messages */
   for(i=0; i<song->numsysexes; i++)
-    sysex_save(song->sysexes[i], node);
+    sysex_save(song->sysexes[i], i, node);
   lb=xmlNewText("\n\n");
   xmlAddChild(doc->xmlRootNode, lb);
 
